#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from selenium.common.exceptions import WebDriverException
from selenium.webdriver.remote import errorhandler

from appium.common.exceptions import NoSuchContextException


class MobileErrorHandler(errorhandler.ErrorHandler):
    def check_response(self, response: Dict) -> None:
        try:
            super().check_response(response)
        except WebDriverException as wde:
            if wde.msg == 'No such context found.':
                raise NoSuchContextException(wde.msg, wde.screen, wde.stacktrace) from wde
            else:
                raise wde
